@@ -88,6 +88,13 @@ def build_parser_bw():
         type=str,
         help=HELP_get_target_nodes,
     )
+    parser_apply.add_argument(
+        "-D",
+        "--no-diff",
+        action='store_false',
+        dest='show_diff',
+        help=_("hide diff for incorrect items when NOT using --interactive"),
+    )
     parser_apply.add_argument(
         "-f",
         "--force",
@@ -817,8 +824,8 @@ bundle:my_bundle  # items in this bundle
                   "If *any* options other than -i are given, *only* the "
                   "tests selected by those options will be run. Otherwise, a "
                   "default selection of tests will be run (that selection may "
-                  "change in future releases). Currently, the default is -IJKM "
-                  "if specific nodes are given and -HIJKMS if testing the "
+                  "change in future releases). Currently, the default is -IJKMp "
+                  "if specific nodes are given and -HIJKMSp if testing the "
                   "entire repo.")
     parser_test = subparsers.add_parser(
         "test",
@@ -956,6 +963,13 @@ bundle:my_bundle  # items in this bundle
         dest='show_all',
         help=_("show correct items as well as incorrect ones"),
     )
+    parser_verify.add_argument(
+        "-D",
+        "--no-diff",
+        action='store_false',
+        dest='show_diff',
+        help=_("hide diff for incorrect items"),
+    )
     bw_verify_p_default = int(environ.get("BW_NODE_WORKERS", "4"))
     parser_verify.add_argument(
         "-p",
