@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from ..utils.text import mark_for_translation as _
 from ..utils.ui import io
 
