@@ -108,7 +108,7 @@ class User(Item):
     @classmethod
     def block_concurrent(cls, node_os, node_os_version):
         # https://github.com/bundlewrap/bundlewrap/issues/367
-        if node_os == 'openbsd':
+        if node_os in ('openbsd', 'freebsd'):
             return [cls.ITEM_TYPE_NAME]
         else:
             return []
@@ -133,20 +133,46 @@ class User(Item):
         return cdict
 
     def fix(self, status):
-        if status.must_be_deleted:
-            self.run("userdel {}".format(self.name), may_fail=True)
+        if self.node.os == 'freebsd':
+            # FreeBSD implements the user{add,mod,del} commands using pw(8).
+            command = "pw "
         else:
-            command = "useradd " if status.must_be_created else "usermod "
+            command = ""
+
+        if status.must_be_deleted:
+            command += "userdel {}"
+            self.run(command.format(self.name), may_fail=True)
+        else:
+            command += "useradd " if status.must_be_created else "usermod "
+            command += f"{self.name} "
+
+            stdin = None
             for attr, option in sorted(_ATTRIBUTE_OPTIONS.items()):
                 if (attr in status.keys_to_fix or status.must_be_created) and \
                         self.attributes[attr] is not None:
                     if attr == 'groups':
                         value = ",".join(self.attributes[attr])
+                    elif attr == 'password_hash' and self.node.os == 'freebsd':
+                        # On FreeBSD, pw useradd/usermod -p sets the password expiry time.
+                        # Using -H <n> we pass the password hash using file descriptor <n> instead.
+                        option = '-H'
+                        value = '0'  # FD 0 = stdin
+                        stdin = self.attributes[attr].encode()
                     else:
                         value = str(self.attributes[attr])
                     command += "{} {} ".format(option, quote(value))
-            command += self.name
-            self.run(command, may_fail=True)
+            self.run(command, data_stdin=stdin, may_fail=True)
+
+    def display_on_create(self, cdict):
+        for attr_name, attr_display_name in _ATTRIBUTE_NAMES.items():
+            if attr_name == attr_display_name:
+                # Don't change anything; the `del` below would
+                # always remove the key entirely!
+                continue
+            if attr_name in cdict:
+                cdict[attr_display_name] = cdict[attr_name]
+                del cdict[attr_name]
+        return cdict
 
     def display_dicts(self, cdict, sdict, keys):
         for attr_name, attr_display_name in _ATTRIBUTE_NAMES.items():
@@ -252,8 +278,8 @@ class User(Item):
                 'hash_method',
                 self.ITEM_ATTRIBUTES['hash_method'],
             )]
-            salt = attributes.get('salt', None)
-            if self.node.os in self.node.OS_FAMILY_BSD:
+            salt = force_text(attributes.get('salt', None))
+            if self.node.os == 'openbsd':
                 attributes['password_hash'] = bcrypt.encrypt(
                     force_text(attributes['password']),
                     rounds=8,  # default rounds for OpenBSD accounts
