@@ -28,6 +28,7 @@ from .metadata import hash_metadata
 from .utils import cached_property, error_context, get_file_contents, names
 from .utils.dicts import (
     dict_to_toml,
+    diff_value,
     hash_statedict,
     set_key_at_path,
     validate_dict,
@@ -116,7 +117,16 @@ def format_node_result(result):
     return ", ".join(output)
 
 
-def handle_apply_result(node, item, status_code, interactive, details=None):
+def handle_apply_result(
+    node,
+    item,
+    status_code,
+    interactive=False,
+    details=None,
+    show_diff=True,
+    created=None,
+    deleted=None,
+):
     if status_code == Item.STATUS_SKIPPED and details in (
         Item.SKIP_REASON_NO_TRIGGER,
         Item.SKIP_REASON_UNLESS,
@@ -126,10 +136,13 @@ def handle_apply_result(node, item, status_code, interactive, details=None):
     formatted_result = format_item_result(
         status_code,
         node.name,
-        item.bundle.name if item.bundle else "",  # dummy items don't have bundles
+        item.bundle.name,
         item.id,
         interactive=interactive,
         details=details,
+        show_diff=show_diff,
+        created=created,
+        deleted=deleted,
     )
     if formatted_result is not None:
         if status_code == Item.STATUS_FAILED:
@@ -150,6 +163,7 @@ def apply_items(
     other_peoples_soft_locks=(),
     workers=1,
     interactive=False,
+    show_diff=True,
 ):
     item_queue = ItemQueue(node)
     # the item queue might contain new generated items (canned actions)
@@ -173,6 +187,7 @@ def apply_items(
                 'my_soft_locks': my_soft_locks,
                 'other_peoples_soft_locks': other_peoples_soft_locks,
                 'interactive': interactive,
+                'show_diff': show_diff,
             },
         }
 
@@ -180,7 +195,7 @@ def apply_items(
         item_id = task_id.split(":", 1)[1]
         item = find_item(item_id, item_queue.pending_items)
 
-        status_code, details = return_value
+        status_code, details, created, deleted = return_value
 
         if status_code == Item.STATUS_FAILED:
             for skipped_item in item_queue.item_failed(item):
@@ -188,7 +203,7 @@ def apply_items(
                     node,
                     skipped_item,
                     Item.STATUS_SKIPPED,
-                    interactive,
+                    interactive=interactive,
                     details=Item.SKIP_REASON_DEP_FAILED,
                 )
                 results.append((skipped_item.id, Item.STATUS_SKIPPED, timedelta(0)))
@@ -208,7 +223,7 @@ def apply_items(
                     node,
                     skipped_item,
                     Item.STATUS_SKIPPED,
-                    interactive,
+                    interactive=interactive,
                     details=skip_reason,
                 )
                 results.append((skipped_item.id, Item.STATUS_SKIPPED, timedelta(0)))
@@ -220,7 +235,16 @@ def apply_items(
                 ),
             ))
 
-        handle_apply_result(node, item, status_code, interactive, details=details)
+        handle_apply_result(
+            node,
+            item,
+            status_code,
+            interactive=interactive,
+            details=details,
+            show_diff=show_diff,
+            created=created,
+            deleted=deleted,
+        )
         io.progress_advance()
         results.append((item.id, status_code, duration))
 
@@ -317,28 +341,59 @@ def format_item_command_results(results):
     return output.lstrip('\n')
 
 
-def format_item_result(result, node, bundle, item, interactive=False, details=None):
-    if details is True:
-        details_text = "({})".format(_("create"))
-    elif details is False:
-        details_text = "({})".format(_("remove"))
-    elif details is None:
+def format_item_result(
+    result,
+    node,
+    bundle,
+    item,
+    interactive=False,
+    details=None,
+    show_diff=True,
+    created=None,
+    deleted=None,
+):
+    if created or deleted or details is None:
         details_text = ""
     elif result == Item.STATUS_SKIPPED:
         details_text = "({})".format(Item.SKIP_REASON_DESC[details])
     else:
-        details_text = "({})".format(", ".join(sorted(details)))
+        details_text = "({})".format(", ".join(sorted(details[2])))
     if result == Item.STATUS_FAILED:
-        return "{x} {node}  {bundle}  {item} {status} {details}".format(
-            bundle=bold(bundle),
-            details=details_text,
-            item=item,
-            node=bold(node),
-            status=red(_("failed")),
-            x=bold(red("✘")),
-        )
+        if created:
+            status = red(_("failed to create"))
+        elif deleted:
+            status = red(_("failed to delete"))
+        else:
+            status = red(_("failed"))
+        if show_diff and not created and not deleted:
+            output = "{x} {node}  {bundle}  {item}  {status}\n".format(
+                bundle=bold(bundle),
+                item=item,
+                node=bold(node),
+                status=status,
+                x=bold(red("✘")),
+            )
+            diff = "\n"
+            for key in sorted(details[2]):
+                diff += diff_value(key, details[1][key], details[0][key]) + "\n"
+            for line in diff.splitlines():
+                output += "{x} {line}\n".format(
+                    line=line,
+                    x=red("│"),
+                )
+            output += red("╵")
+            return output
+        else:
+            return "{x} {node}  {bundle}  {item}  {status} {details}".format(
+                bundle=bold(bundle),
+                details=details_text,
+                item=item,
+                node=bold(node),
+                status=status,
+                x=bold(red("✘")),
+            )
     elif result == Item.STATUS_ACTION_SUCCEEDED:
-        return "{x} {node}  {bundle}  {item} {status}".format(
+        return "{x} {node}  {bundle}  {item}  {status}".format(
             bundle=bold(bundle),
             item=item,
             node=bold(node),
@@ -346,7 +401,7 @@ def format_item_result(result, node, bundle, item, interactive=False, details=No
             x=bold(green("✓")),
         )
     elif result == Item.STATUS_SKIPPED:
-        return "{x} {node}  {bundle}  {item} {status} {details}".format(
+        return "{x} {node}  {bundle}  {item}  {status} {details}".format(
             bundle=bold(bundle),
             details=details_text,
             item=item,
@@ -355,14 +410,42 @@ def format_item_result(result, node, bundle, item, interactive=False, details=No
             status=yellow(_("skipped")),
         )
     elif result == Item.STATUS_FIXED:
-        return "{x} {node}  {bundle}  {item} {status} {details}".format(
-            bundle=bold(bundle),
-            details=details_text,
-            item=item,
-            node=bold(node),
-            x=bold(green("✓")),
-            status=green(_("fixed")),
-        )
+        if created:
+            status = green(_("created"))
+        elif deleted:
+            status = green(_("deleted"))
+        else:
+            status = green(_("fixed"))
+        if not interactive and show_diff:
+            output = "{x} {node}  {bundle}  {item}  {status}\n".format(
+                bundle=bold(bundle),
+                item=item,
+                node=bold(node),
+                x=bold(green("✓")),
+                status=status,
+            )
+            diff = "\n"
+            if created or deleted:
+                for key, value in sorted(details.items()):
+                    diff += f"{bold(key)}  {value}\n"
+            else:
+                for key in sorted(details[2]):
+                    diff += diff_value(key, details[1][key], details[0][key]) + "\n"
+            for line in diff.splitlines():
+                output += "{x} {line}\n".format(
+                    line=line,
+                    x=green("│"),
+                )
+            output += green("╵")
+            return output
+        else:
+            return "{x} {node}  {bundle}  {item}  {status}".format(
+                bundle=bold(bundle),
+                item=item,
+                node=bold(node),
+                x=bold(green("✓")),
+                status=status,
+            )
 
 
 class Node:
@@ -547,6 +630,7 @@ class Node:
         autoonly_selector="",
         interactive=False,
         force=False,
+        show_diff=True,
         skip_list=tuple(),
         workers=4,
     ):
@@ -619,6 +703,7 @@ class Node:
                         other_peoples_soft_locks=lock.other_peoples_soft_locks,
                         workers=workers,
                         interactive=interactive,
+                        show_diff=show_diff,
                     )
             except NodeLockedException as e:
                 if not interactive:
@@ -794,14 +879,14 @@ class Node:
             wrapper_outer=self.cmd_wrapper_outer,
         )
 
-    def verify(self, show_all=False, workers=4):
+    def verify(self, show_all=False, show_diff=True, workers=4):
         result = []
         start = datetime.now()
 
         if not self.items:
             io.stdout(_("{x} {node}  has no items").format(node=bold(self.name), x=yellow("!")))
         else:
-            result = verify_items(self, show_all=show_all, workers=workers)
+            result = verify_items(self, show_all=show_all, show_diff=show_diff, workers=workers)
 
         return {
             'good': result.count(True),
@@ -846,7 +931,7 @@ for attr, default in GROUP_ATTR_DEFAULTS.items():
     setattr(Node, attr, build_attr_property(attr, default))
 
 
-def verify_items(node, show_all=False, workers=1):
+def verify_items(node, show_all=False, show_diff=True, workers=1):
     items = []
     for item in node.items:
         if not item.triggered:
@@ -924,22 +1009,46 @@ def verify_items(node, show_all=False, workers=1):
 
     def handle_result(task_id, return_value, duration):
         io.progress_advance()
-        unless_result, item_status = return_value
+        unless_result, item_status, display = return_value
         node_name, bundle_name, item_id = task_id.split(":", 2)
         if not unless_result and not item_status.correct:
             if item_status.must_be_created:
-                details_text = _("create")
+                details_text = red(_("missing"))
             elif item_status.must_be_deleted:
-                details_text = _("remove")
+                details_text = red(_("found"))
+            elif show_diff:
+                details_text = ""
             else:
-                details_text = ", ".join(sorted(item_status.display_keys_to_fix))
-            io.stderr("{x} {node}  {bundle}  {item} ({details})".format(
-                bundle=bold(bundle_name),
-                details=details_text,
-                item=item_id,
-                node=bold(node_name),
-                x=red("✘"),
-            ))
+                details_text = ", ".join(sorted(display[2]))
+            if show_diff:
+                diff = "\n"
+                if item_status.must_be_created or item_status.must_be_deleted:
+                    for key, value in sorted(display.items()):
+                        diff += f"{bold(key)}  {value}\n"
+                else:
+                    for key in sorted(display[2]):
+                        diff += diff_value(key, display[1][key], display[0][key]) + "\n"
+                output = "{x} {node}  {bundle}  {item}  {details}\n".format(
+                    bundle=bold(bundle_name),
+                    details=details_text,
+                    item=item_id,
+                    node=bold(node_name),
+                    x=red("✘"),
+                )
+                for line in diff.splitlines():
+                    output += "{x} {line}\n".format(
+                        line=line,
+                        x=red("│"),
+                    )
+                io.stderr(output + red("╵"))
+            else:
+                io.stderr("{x} {node}  {bundle}  {item}  {details}".format(
+                    bundle=bold(bundle_name),
+                    details=details_text,
+                    item=item_id,
+                    node=bold(node_name),
+                    x=red("✘"),
+                ))
             return False
         else:
             if show_all:
