@@ -1,11 +1,5 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from base64 import b64encode, urlsafe_b64decode
-try:
-    from configparser import SafeConfigParser
-except ImportError:  # Python 2
-    from ConfigParser import SafeConfigParser
+from configparser import ConfigParser
 import hashlib
 import hmac
 from os import environ
@@ -64,7 +58,7 @@ def random(seed):
                 yield character
 
 
-class SecretProxy(object):
+class SecretProxy:
     @staticmethod
     def random_key():
         """
@@ -75,7 +69,6 @@ class SecretProxy(object):
     def __init__(self, repo):
         self.repo = repo
         self.keys = self._load_keys()
-        self._call_log = {}
 
     def _decrypt(self, cryptotext=None, key=None):
         """
@@ -243,7 +236,7 @@ class SecretProxy(object):
         return random(h.digest())
 
     def _load_keys(self):
-        config = SafeConfigParser()
+        config = ConfigParser()
         secrets_file = join(self.repo.path, FILENAME_SECRETS)
         try:
             config.read(secrets_file)
@@ -257,6 +250,7 @@ class SecretProxy(object):
 
     def decrypt(self, cryptotext, key=None):
         return Fault(
+            'bw secrets decrypt',
             self._decrypt,
             cryptotext=cryptotext,
             key=key,
@@ -264,6 +258,7 @@ class SecretProxy(object):
 
     def decrypt_file(self, source_path, key=None):
         return Fault(
+            'bw secrets decrypt_file',
             self._decrypt_file,
             source_path=source_path,
             key=key,
@@ -271,6 +266,7 @@ class SecretProxy(object):
 
     def decrypt_file_as_base64(self, source_path, key=None):
         return Fault(
+            'bw secrets decrypt_file_as_base64',
             self._decrypt_file_as_base64,
             source_path=source_path,
             key=key,
@@ -322,9 +318,8 @@ class SecretProxy(object):
     def human_password_for(
         self, identifier, digits=2, key='generate', per_word=3, words=4,
     ):
-        self._call_log.setdefault(identifier, 0)
-        self._call_log[identifier] += 1
         return Fault(
+            'bw secrets human_password_for',
             self._generate_human_password,
             identifier=identifier,
             digits=digits,
@@ -334,9 +329,8 @@ class SecretProxy(object):
         )
 
     def password_for(self, identifier, key='generate', length=32, symbols=False):
-        self._call_log.setdefault(identifier, 0)
-        self._call_log[identifier] += 1
         return Fault(
+            'bw secrets password_for',
             self._generate_password,
             identifier=identifier,
             key=key,
@@ -346,6 +340,7 @@ class SecretProxy(object):
 
     def random_bytes_as_base64_for(self, identifier, key='generate', length=32):
         return Fault(
+            'bw secrets random_bytes_as_base64',
             self._generate_random_bytes_as_base64,
             identifier=identifier,
             key=key,
