@@ -10,6 +10,7 @@ import stat
 from sys import stderr, stdout
 from tempfile import mkstemp
 
+from passlib.hash import apr_md5_crypt
 from requests import get
 
 from ..exceptions import DontCache, FaultUnavailable
@@ -113,7 +114,7 @@ class Fault:
 
         for key, value in sorted(kwargs.items()):
             self.id_list.append(hash(key))
-            self.id_list.append(hash(value))
+            self.id_list.append(_recursive_hash(value))
 
         self._available = None
         self._exc = None
@@ -155,7 +156,19 @@ class Fault:
         return len(self.value)
 
     def __lt__(self, other):
-        return self.value < other.value
+        if isinstance(other, Fault):
+            return self.value < other.value
+        else:
+            return self.value < other
+
+    def __gt__(self, other):
+        if isinstance(other, Fault):
+            return self.value > other.value
+        else:
+            return self.value > other
+
+    def __repr__(self):
+        return f"<Fault: {self.id_list}>"
 
     def __str__(self):
         return str(self.value)
@@ -170,6 +183,17 @@ class Fault:
             return format_string.format(self.value)
         return Fault(self.id_list + ['format_into ' + format_string], callback)
 
+    def as_htpasswd_entry(self, username):
+        def callback():
+            return '{}:{}'.format(
+                username,
+                apr_md5_crypt.encrypt(
+                    self.value,
+                    salt=hashlib.sha512(self.id_list[0].encode('utf-8')).hexdigest()[:8],
+                ),
+            )
+        return Fault(self.id_list + ['as_htpasswd_entry ' + username], callback)
+
     @property
     def is_available(self):
         self._resolve()
@@ -204,6 +228,25 @@ for method_name in (
     setattr(Fault, method_name, _make_method_callback(method_name))
 
 
+def _recursive_hash(obj):
+    hashes = []
+    if isinstance(obj, list):
+        for i in obj:
+            hashes.append(_recursive_hash(i))
+        return hash(tuple(hashes))
+    elif isinstance(obj, set):
+        for i in sorted(obj):
+            hashes.append(_recursive_hash(i))
+        return hash(tuple(hashes))
+    elif isinstance(obj, dict):
+        for k, v in sorted(obj.items()):
+            hashes.append(hash(k))
+            hashes.append(_recursive_hash(v))
+        return hash(tuple(hashes))
+    else:
+        return hash(obj)
+
+
 def get_file_contents(path):
     with error_context(path=path):
         with open(path, 'rb') as f:
@@ -218,6 +261,18 @@ def hash_local_file(path):
     return sha1(get_file_contents(path))
 
 
+def list_starts_with(list_a, list_b):
+    """
+    Returns True if list_a starts with list_b.
+    """
+    list_a = tuple(list_a)
+    list_b = tuple(list_b)
+    try:
+        return list_a[:len(list_b)] == list_b
+    except IndexError:
+        return False
+
+
 def names(obj_list):
     """
     Iterator over the name properties of a given list of objects.
