@@ -4,6 +4,12 @@ from hashlib import sha1
 from json import dumps, JSONEncoder
 
 from tomlkit import document as toml_document
+from tomlkit.items import (
+    Bool as TOMLBool,
+    Float as TOMLFloat,
+    Integer as TOMLInteger,
+    String as TOMLString,
+)
 
 from . import Fault
 from .text import bold, green, red
@@ -58,13 +64,13 @@ def diff_keys(sdict1, sdict2):
     only exists in the second one, it is disregarded.
     """
     if sdict1 is None:
-        return []
+        return set()
     if sdict2 is None:
-        return sdict1.keys()
-    differing_keys = []
+        return set(sdict1.keys())
+    differing_keys = set()
     for key, value in sdict1.items():
         if value != sdict2[key]:
-            differing_keys.append(key)
+            differing_keys.add(key)
     return differing_keys
 
 
@@ -142,7 +148,7 @@ def diff_value_text(title, value1, value2):
         elif line.startswith("-"):
             line = red(line)
         output += line + suffix + "\n"
-    return output
+    return output.rstrip("\n")
 
 
 TYPE_DIFFS = {
@@ -154,16 +160,23 @@ TYPE_DIFFS = {
     set: diff_value_list,
     str: diff_value_text,
     tuple: diff_value_list,
+    TOMLBool: diff_value_bool,
+    TOMLFloat: diff_value_int,
+    TOMLInteger: diff_value_int,
+    TOMLString: diff_value_text,
 }
 
 
 def diff_value(title, value1, value2):
-    value_type = type(value1)
-    assert value_type == type(value2), "cannot compare {} with {}".format(
-        repr(value1),
-        repr(value2),
-    )
-    diff_func = TYPE_DIFFS[value_type]
+    diff_func = TYPE_DIFFS[type(value1)]
+    diff_func2 = TYPE_DIFFS[type(value2)]
+    if diff_func != diff_func2:
+        raise TypeError(_("cannot compare {} ({}) with {} ({})").format(
+            repr(value1),
+            type(value1),
+            repr(value2),
+            type(value2),
+        ))
     return diff_func(title, value1, value2)
 
 
