@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from datetime import datetime, timedelta
 from io import BytesIO
 from os import environ
@@ -44,11 +41,6 @@ def cyan(text):
     return "\033[36m{}\033[0m".format(text)
 
 
-@ansi_wrapper
-def inverse(text):
-    return "\033[0m\033[7m{}\033[0m".format(text)
-
-
 @ansi_wrapper
 def italic(text):
     return "\033[3m{}\033[0m".format(text)
@@ -122,7 +114,6 @@ def error_summary(errors):
 def force_text(data):
     """
     Try to return a text aka unicode object from the given data.
-    Also has Python 2/3 compatibility baked in. Oh the humanity.
     """
     if isinstance(data, bytes):
         return data.decode('utf-8', 'replace')
@@ -187,7 +178,7 @@ def wrap_question(title, body, question, prefix=""):
     return output
 
 
-class LineBuffer(object):
+class LineBuffer:
     def __init__(self, target):
         self.buffer = b""
         self.record = BytesIO()
@@ -258,3 +249,25 @@ def parse_duration(duration):
         else:
             raise ValueError(_("{} is not a valid duration string").format(repr(duration)))
     return timedelta(days=days, seconds=seconds)
+
+
+def toml_clean(s):
+    """
+    Removes duplicate sections from TOML, e.g.:
+
+        [foo]     <--- this line will be removed since it's redundant
+        [foo.bar]
+        baz = 1
+    """
+    lines = list(s.splitlines())
+    result = []
+    previous = ""
+    for line in lines.copy():
+        if line.startswith("[") and line.endswith("]"):
+            if line[1:].startswith(previous + "."):
+                result.pop()
+            previous = line[1:-1]
+        else:
+            previous = ""
+        result.append(line)
+    return "\n".join(result) + "\n"
