@@ -97,7 +97,7 @@ def sigquit_handler(*args, **kwargs):
 
 def spinner():
     while True:
-        for c in "⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏":
+        for c in "⠁⠈⠐⠠⢀⡀⠄⠂":
             yield c
 
 
@@ -106,10 +106,16 @@ def page_lines(lines):
     View the given list of Unicode lines in a pager (e.g. `less`).
     """
     lines = list(lines)
-    if TTY:
+    line_width = max([len(ansi_clean(line)) for line in lines])
+    if (
+        TTY and (
+            line_width > get_terminal_size().columns or
+            len(lines) > get_terminal_size().lines
+        )
+    ):
         write_to_stream(STDOUT_WRITER, SHOW_CURSOR)
         env = environ.copy()
-        env["LESS"] = env.get("LESS", "") + " -FR"
+        env["LESS"] = env.get("LESS", "") + " -R"
         pager = Popen(
             [environ.get("PAGER", "/usr/bin/less")],
             env=env,
@@ -338,6 +344,7 @@ class IOManager:
 
     def job_wrapper(self, job_text):
         def outer_wrapper(wrapped_function):
+            @wraps(wrapped_function)
             def inner_wrapper(*args, **kwargs):
                 with self.job(job_text.format(*args, **kwargs)):
                     return wrapped_function(*args, **kwargs)
