@@ -6,7 +6,6 @@ from bundlewrap.utils.testing import make_repo, run
 def test_empty(tmpdir):
     make_repo(tmpdir)
     stdout, stderr, rcode = run("bw test", path=str(tmpdir))
-    assert stdout == b""
     assert stderr == b""
     assert rcode == 0
 
@@ -53,12 +52,14 @@ def test_circular_dep_direct(tmpdir):
         },
         bundles={
             "bundle1": {
-                "pkg_apt": {
-                    "foo": {
-                        'needs': ["pkg_apt:bar"],
-                    },
-                    "bar": {
-                        'needs': ["pkg_apt:foo"],
+                'items': {
+                    "pkg_apt": {
+                        "foo": {
+                            'needs': ["pkg_apt:bar"],
+                        },
+                        "bar": {
+                            'needs': ["pkg_apt:foo"],
+                        },
                     },
                 },
             },
@@ -77,15 +78,17 @@ def test_circular_dep_indirect(tmpdir):
         },
         bundles={
             "bundle1": {
-                "pkg_apt": {
-                    "foo": {
-                        'needs': ["pkg_apt:bar"],
-                    },
-                    "bar": {
-                        'needs': ["pkg_apt:baz"],
-                    },
-                    "baz": {
-                        'needs': ["pkg_apt:foo"],
+                'items': {
+                    "pkg_apt": {
+                        "foo": {
+                            'needs': ["pkg_apt:bar"],
+                        },
+                        "bar": {
+                            'needs': ["pkg_apt:baz"],
+                        },
+                        "baz": {
+                            'needs': ["pkg_apt:foo"],
+                        },
                     },
                 },
             },
@@ -104,9 +107,11 @@ def test_circular_dep_self(tmpdir):
         },
         bundles={
             "bundle1": {
-                "pkg_apt": {
-                    "foo": {
-                        'needs': ["pkg_apt:foo"],
+                'items': {
+                    "pkg_apt": {
+                        "foo": {
+                            'needs': ["pkg_apt:foo"],
+                        },
                     },
                 },
             },
@@ -125,11 +130,13 @@ def test_unknown_tag(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/foo": {
-                        'content': "none",
-                        'needs': {
-                            "tag:bar",
+                'items': {
+                    "files": {
+                        "/foo": {
+                            'content': "none",
+                            'needs': {
+                                "tag:bar",
+                            },
                         },
                     },
                 },
@@ -149,9 +156,11 @@ def test_circular_trigger_self(tmpdir):
         },
         bundles={
             "bundle1": {
-                "pkg_apt": {
-                    "foo": {
-                        'triggers': ["pkg_apt:foo"],
+                'items': {
+                    "pkg_apt": {
+                        "foo": {
+                            'triggers': ["pkg_apt:foo"],
+                        },
                     },
                 },
             },
@@ -170,9 +179,11 @@ def test_file_invalid_attribute(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/foo": {
-                        "potato": "yes",
+                'items': {
+                    "files": {
+                        "/foo": {
+                            "potato": "yes",
+                        },
                     },
                 },
             },
@@ -191,10 +202,12 @@ def test_file_template_error(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/foo": {
-                        'content_type': 'mako',
-                        'content': "${broken",
+                'items': {
+                    "files": {
+                        "/foo": {
+                            'content_type': 'mako',
+                            'content': "${broken",
+                        },
                     },
                 },
             },
@@ -604,6 +617,52 @@ def foo(metadata):
     assert b"foo/bar" in stderr
 
 
+def test_reactor_provides_ok(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor.provides("foo")
+def foo(metadata):
+    return {"foo": 1}
+""")
+    stdout, stderr, rcode = run("bw test -p", path=str(tmpdir))
+    assert rcode == 0
+
+
+def test_reactor_provides_violated(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': {"bundle1"},
+            },
+        },
+        bundles={
+            "bundle1": {},
+        },
+    )
+    with open(join(str(tmpdir), "bundles", "bundle1", "metadata.py"), 'w') as f:
+        f.write(
+"""@metadata_reactor.provides("foo")
+def foo(metadata):
+    return {"bar": 1}
+""")
+    stdout, stderr, rcode = run("bw test -p", path=str(tmpdir))
+    assert rcode == 1
+    assert "foo" in stderr.decode()
+    assert "bar" in stderr.decode()
+
+
 def test_fault_missing(tmpdir):
     make_repo(
         tmpdir,
@@ -614,10 +673,12 @@ def test_fault_missing(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/foo": {
-                        'content_type': 'mako',
-                        'content': "${repo.vault.decrypt('bzzt', key='unavailable')}",
+                'items': {
+                    "files": {
+                        "/foo": {
+                            'content_type': 'mako',
+                            'content': "${repo.vault.decrypt('bzzt', key='unavailable')}",
+                        },
                     },
                 },
             },
@@ -689,9 +750,11 @@ n = randint(1, 99999)
 
 @metadata_reactor
 def test(metadata):
-    return {'test': n}
+    return {'findme': n}
 """)
-    assert run("bw test -m 3", path=str(tmpdir))[2] == 1
+    stdout, stderr, rcode = run("bw test -m 3", path=str(tmpdir))
+    assert rcode == 1
+    assert b"findme" in stderr
 
 
 def test_config_determinism_ok(tmpdir):
@@ -704,10 +767,12 @@ def test_config_determinism_ok(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/test": {
-                        'content': "1",
-                        'content_type': 'mako',
+                'items': {
+                    "files": {
+                        "/test": {
+                            'content': "1",
+                            'content_type': 'mako',
+                        },
                     },
                 },
             },
@@ -726,10 +791,12 @@ def test_config_determinism_broken(tmpdir):
         },
         bundles={
             "bundle1": {
-                "files": {
-                    "/test": {
-                        'content': "<% from random import randint %>\n${randint(1, 99999)\n}",
-                        'content_type': 'mako',
+                'items': {
+                    "files": {
+                        "/test": {
+                            'content': "<% from random import randint %>\n${randint(1, 99999)\n}",
+                            'content_type': 'mako',
+                        },
                     },
                 },
             },
@@ -779,14 +846,16 @@ def test_group_user_dep_deleted(tmpdir):
         },
         bundles={
             "bundle1": {
-                "users": {
-                    "user1": {
-                        'groups': ["group1"],
+                'items': {
+                    "users": {
+                        "user1": {
+                            'groups': ["group1"],
+                        },
                     },
-                },
-                "groups": {
-                    "group1": {
-                        'delete': True,
+                    "groups": {
+                        "group1": {
+                            'delete': True,
+                        },
                     },
                 },
             },
@@ -806,11 +875,13 @@ def test_group_user_dep_ok(tmpdir):
         },
         bundles={
             "bundle1": {
-                "users": {
-                    "user1": {},
-                },
-                "groups": {
-                    "group1": {'delete': True},
+                'items': {
+                    "users": {
+                        "user1": {},
+                    },
+                    "groups": {
+                        "group1": {'delete': True},
+                    },
                 },
             },
         },
@@ -828,14 +899,16 @@ def test_group_user_dep_deleted_gid(tmpdir):
         },
         bundles={
             "bundle1": {
-                "users": {
-                    "user1": {
-                        'gid': "group1",
+                'items': {
+                    "users": {
+                        "user1": {
+                            'gid': "group1",
+                        },
                     },
-                },
-                "groups": {
-                    "group1": {
-                        'delete': True,
+                    "groups": {
+                        "group1": {
+                            'delete': True,
+                        },
                     },
                 },
             },
@@ -854,17 +927,21 @@ def test_reverse_dummy_dep(tmpdir):
         },
         bundles={
             "bundle1": {
-                'files': {
-                    "/test": {
-                        'content': "test",
+                'items': {
+                    'files': {
+                        "/test": {
+                            'content': "test",
+                        },
                     },
                 },
             },
             "bundle2": {
-                'files': {
-                    "/test2": {
-                        'content': "test",
-                        'needed_by': ["bundle:bundle1"],
+                'items': {
+                    'files': {
+                        "/test2": {
+                            'content': "test",
+                            'needed_by': ["bundle:bundle1"],
+                        },
                     },
                 },
             },
@@ -873,3 +950,92 @@ def test_reverse_dummy_dep(tmpdir):
 
     stdout, stderr, rcode = run("bw test", path=str(tmpdir))
     assert rcode == 0
+
+
+def test_bundlepy_tag_loop(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': ["bundle1"],
+            },
+        },
+        bundles={
+            "bundle1": {
+                'attrs': {
+                    'tags': {
+                        "a": {
+                            'needs': {"tag:b"},
+                        },
+                        "b": {
+                            'needs': {"tag:a"},
+                        },
+                    },
+                },
+                'items': {
+                    'actions': {
+                        "one": {
+                            'command': "true",
+                            'tags': {"a"},
+                        },
+                        "two": {
+                            'command': "true",
+                            'tags': {"b"},
+                        },
+                    },
+                },
+            },
+        },
+    )
+    stdout, stderr, rcode = run("bw test -I", path=str(tmpdir))
+    assert rcode == 1
+    assert "action:one" in stderr.decode()
+    assert "action:two" in stderr.decode()
+
+
+def test_bundlepy_tag_loop2(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'bundles': ["bundle1"],
+            },
+        },
+        bundles={
+            "bundle1": {
+                'attrs': {
+                    'tags': {
+                        "one": {
+                            'needed_by': {"tag:two"},
+                        },
+                        "two": {
+                            'needed_by': {"action:late"},
+                        },
+                    },
+                },
+                'items': {
+                    'actions': {
+                        "early": {
+                            'command': "true",
+                            'needed_by': {"tag:one"},
+                        },
+                        "fill_tag_one": {
+                            'command': "true",
+                            'tags': {"one"},
+                        },
+                        "fill_tag_two": {
+                            'command': "true",
+                            'tags': {"two"},
+                        },
+                        "late": {
+                            'command': "true",
+                            'needed_by': {"action:early"},  # this makes the loop
+                        },
+                    },
+                },
+            },
+        },
+    )
+    stdout, stderr, rcode = run("bw test -I", path=str(tmpdir))
+    assert rcode == 1
+    assert "action:late" in stderr.decode()
